@@ -42,6 +42,12 @@ pub fn trivia_contains_newline<'a>(trivia_vec: impl Iterator<Item = &'a Token>)
     false
 }
 
+/// Determines whether a particular node spans over multiple lines
+pub fn spans_multiple_lines<T: std::fmt::Display>(item: &T) -> bool {
+    let string = format!("{}", item);
+    string.lines().count() > 1
+}
+
 pub fn can_hang_expression(expression: &Expression) -> bool {
     match expression {
         Expression::Parentheses { expression, .. } => can_hang_expression(expression),
@@ -410,11 +416,10 @@ pub fn take_expression_trailing_comments(expression: &Expression) -> (Expression
     let trailing_comments = get_expression_trailing_trivia(expression)
         .iter()
         .filter(|token| trivia_is_comment(token))
-        .map(|x| {
+        .flat_map(|x| {
             // Prepend a single space beforehand
             vec![Token::new(TokenType::spaces(1)), x.to_owned()]
         })
-        .flatten()
         .collect();
 
     (
@@ -492,6 +497,16 @@ fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>)
             let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
             (TypeInfo::Basic(token), trailing_trivia)
         }
+        TypeInfo::String(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (TypeInfo::String(token), trailing_trivia)
+        }
+        TypeInfo::Boolean(token) => {
+            let trailing_trivia = token.trailing_trivia().map(|x| x.to_owned()).collect();
+            let token = token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (TypeInfo::Boolean(token), trailing_trivia)
+        }
         TypeInfo::Callback {
             generics,
             parentheses,
@@ -532,6 +547,11 @@ fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>)
                 trailing_trivia,
             )
         }
+        TypeInfo::GenericPack { name, ellipse } => {
+            let trailing_trivia = ellipse.trailing_trivia().map(|x| x.to_owned()).collect();
+            let ellipse = ellipse.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (TypeInfo::GenericPack { name, ellipse }, trailing_trivia)
+        }
         TypeInfo::Intersection {
             left,
             ampersand,
@@ -648,6 +668,11 @@ fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>)
                 trailing_trivia,
             )
         }
+        TypeInfo::VariadicPack { ellipse, name } => {
+            let trailing_trivia = name.trailing_trivia().map(|x| x.to_owned()).collect();
+            let name = name.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
+            (TypeInfo::VariadicPack { ellipse, name }, trailing_trivia)
+        }
         other => panic!("unknown node {:?}", other),
     }
 }
